###################################################
### chunk number 1: preliminaries
###################################################
online <- FALSE ## if set to FALSE the local copy of MSFT.rda
                ## is used instead of get.hist.quote()
options(prompt = "R> ")


###################################################
### chunk number 2: SynthOptimalk eval=FALSE
###################################################
## library("nnc")
## library("MASS")
## X <- synth.tr[,1:2]
## y <- factor(synth.tr[,3])
## KMAX<-100
## d<-numeric(KMAX)
## for (k in 1:KMAX){
##     z <- nnc(k = k, X = X, Y = y)
##     d[k]<- deviance(glm.fit(x = z, y = y, family = binomial(link = "logit")))
##     }
## khat <- which.min(d)    
## plot(d, xlab="k", ylab="deviance")
## points(khat, d[khat], col="red", pch=16, cex=1.4)
## title(sub=bquote(hat(k) == .(khat)))


###################################################
### chunk number 3: SynthOptimalk-repeat
###################################################
library("nnc")
library("MASS")
X <- synth.tr[,1:2]
y <- factor(synth.tr[,3])
KMAX<-100
d<-numeric(KMAX)
for (k in 1:KMAX){
    z <- nnc(k = k, X = X, Y = y)
    d[k]<- deviance(glm.fit(x = z, y = y, family = binomial(link = "logit")))
    }
khat <- which.min(d)    
plot(d, xlab="k", ylab="deviance")
points(khat, d[khat], col="red", pch=16, cex=1.4)
title(sub=bquote(hat(k) == .(khat)))


###################################################
### chunk number 4: TiesKis66 eval=FALSE
###################################################
## XTest <- synth.te[,1:2]
## yTest <- factor(synth.te[,3])
## REP<-100
## K<-66
## e<-numeric(REP)
## for (i in 1:REP){
##     yhat<-knn(train=X, test=XTest, cl=y, k = K, use.all=TRUE)
##     e[i] <- mean(yhat!=yTest)
##     }
## hist(e, main="Distribution of error rate", xlab="Error rate")


###################################################
### chunk number 5: TiesKis66-repeat
###################################################
XTest <- synth.te[,1:2]
yTest <- factor(synth.te[,3])
REP<-100
K<-66
e<-numeric(REP)
for (i in 1:REP){
    yhat<-knn(train=X, test=XTest, cl=y, k = K, use.all=TRUE)
    e[i] <- mean(yhat!=yTest)
    }
hist(e, main="Distribution of error rate", xlab="Error rate")


###################################################
### chunk number 6: TiesKis67
###################################################
K<-67
for (i in 1:REP){
    yhat<-knn(train=X, test=XTest, cl=y, k = 67)
    e[i] <- mean(yhat!=yTest)
    }
summary(e)


###################################################
### chunk number 7: Iriskhat eval=FALSE
###################################################
## khat(iris[,1:4], iris[,5])


###################################################
### chunk number 8: Iriskhat-repeat
###################################################
khat(iris[,1:4], iris[,5])


###################################################
### chunk number 9: irisParallel eval=FALSE
###################################################
## library("lattice")
## parallel(~iris[,1:4], groups=iris[,5], auto.key=list(space="top", columns=3))


###################################################
### chunk number 10: DefineOneItn
###################################################
OneItn<-function(){
	Y<-iris[,5]
	X<-iris[,1:4]
	Xset <- iris[iris$Species=="setosa", 1:4]
	Xver <- iris[iris$Species=="versicolor", 1:4]
	Xvir <- iris[iris$Species=="virginica", 1:4]
	Yset <- iris[iris$Species=="setosa", 5]
	Yver <- iris[iris$Species=="versicolor", 5]
	Yvir <- iris[iris$Species=="virginica", 5]
	n<-nrow(Xset)
	fold<-5
	nTest<-floor(n/fold)
	NREP<-100
	kmax<-80
    jset<-sample(1:n, size=nTest)
    jver<-sample(1:n, size=nTest)
    jvir<-sample(1:n, size=nTest)
    indsetTest<-rep(FALSE, n)
    indsetTest[jset]<-TRUE
    indverTest<-rep(FALSE, n)
    indverTest[jver]<-TRUE
    indvirTest<-rep(FALSE, n)
    indvirTest[jvir]<-TRUE
    YsetTrain<-Yset[!indsetTest]
    YsetTest<-Yset[indsetTest]
    XsetTrain <- Xset[!indsetTest,]
    XsetTest <- Xset[indsetTest,]
    YverTrain<-Yver[!indverTest]
    YverTest<-Yver[indverTest]
    XverTrain <- Xver[!indverTest,]
    XverTest <- Xver[indverTest,]    
    YvirTrain<-Yvir[!indvirTest]
    YvirTest<-Yvir[indvirTest]
    XvirTrain <- Xvir[!indvirTest,]
    XvirTest <- Xvir[indvirTest,]     
    XTrain <- rbind(XsetTrain, XverTrain, XvirTrain)
    YTrain <- c(YsetTrain, YverTrain, YvirTrain)
    XTest <- rbind(XsetTest, XverTest, XvirTest)
    YTest <- c(YsetTest, YverTest, YvirTest)        
    train.LDA<-lda(XTrain, YTrain)
    yfitLDA<-predict(train.LDA, newdata=XTest, dimen=1)$class
    etaFLDA<-mean(as.numeric(yfitLDA!=YTest))
    kopt <- khat(XTrain, YTrain, kmax, plot=FALSE)
    ans<-knn(train=XTrain, test=XTest, k=kopt, cl=YTrain)
    etaKNN <- mean(as.numeric(ans!=YTest))
    eta<-c(etaFLDA,etaKNN)
    names(eta)<-c("LDA","kNN")
	eta
} 


###################################################
### chunk number 11: RunHonestCV
###################################################
library("nnc")
StartTime<-proc.time()[1]
NREP<-5
eta<-matrix(numeric(NREP*2), ncol=2)
for (i in 1:NREP)
    eta[i,] <- OneItn()
EndTime<-proc.time()[1]
TotTime<-EndTime-StartTime
e<-apply(eta, MARGIN=2, FUN=mean)
e
TotTime


